# Quick tests for the markup templatetags (django_markwhat)
import re
import unittest

from django.template import Template, Context
from django.utils.html import escape

try:
    import textile
except ImportError:
    textile = None

try:
    import markdown

    markdown_version = getattr(markdown, "version_info", 0)
except ImportError:
    markdown = None

try:
    import commonmark
except ImportError:
    commonmark = None

try:
    import docutils
except ImportError:
    docutils = None


class Templates(unittest.TestCase):
    textile_content = """Paragraph 1

Paragraph 2 with "quotes" and @code@"""

    markdown_content = """Paragraph 1

## An h2"""

    markdown_content_with_html_code = """Paragraph 1

## An h2

```
    <video width="320" height="240" controls>
        <source src="movie.mp4" type="video/mp4">
        <source src="movie.ogg" type="video/ogg">
    </video>
```
"""

    markdown_content_with_iframe_code = """Paragraph 1

## An h2

```
    <iframe src="http://example.com"></iframe>
```
"""

    rest_content = """Paragraph 1

Paragraph 2 with a link_

.. _link: http://www.example.com/"""

    @unittest.skipUnless(textile, 'texttile not installed')
    def test_textile(self):
        t = Template("{% load markup %}{{ textile_content|textile }}")
        rendered = t.render(Context(
            {'textile_content': self.textile_content})).strip()
        self.assertEqual(rendered.replace('\t', ''), """<p>Paragraph 1</p>

<p>Paragraph 2 with &#8220;quotes&#8221; and <code>code</code></p>""")

    @unittest.skipIf(textile, 'texttile is installed')
    def test_no_textile(self):
        t = Template("{% load markup %}{{ textile_content|textile }}")
        rendered = t.render(Context(
            {'textile_content': self.textile_content})).strip()
        self.assertEqual(rendered, escape(self.textile_content))

    @unittest.skipUnless(markdown, 'markdown not installed')
    def test_markdown(self):
        t = Template("{% load markup %}{{ markdown_content|markdown }}")
        rendered = t.render(Context({
            'markdown_content': self.markdown_content
        })).strip()
        pattern = re.compile("""<p>Paragraph 1\s*</p>\s*<h2>\s*An h2</h2>""")
        self.assertTrue(pattern.match(rendered))

    @unittest.skipUnless(markdown, 'markdown not installed')
    def test_markdown_html_code(self):
        t = Template("{% load markup %}{{ markdown_content|markdown }}")
        rendered = t.render(Context({
            'markdown_content': self.markdown_content_with_html_code
        })).strip()
        pattern = re.compile(
            '<p>Paragraph 1\s*</p>\s*<h2>\s*An h2</h2>' +
            '\s*<p><code>\s*&lt;video width="320"'
        )
        self.assertTrue(pattern.match(rendered))

    @unittest.skipUnless(markdown, 'markdown not installed')
    def test_markdown_html_iframe_code(self):
        t = Template("{% load markup %}{{ markdown_content|markdown }}")
        rendered = t.render(Context({
            'markdown_content': self.markdown_content_with_iframe_code
        })).strip()
        pattern = re.compile(
            '<p>Paragraph 1\s*</p>\s*<h2>\s*An h2</h2>' +
            '\s*<p><code>\s*&lt;iframe src="http://example.com"&gt;' +
            '&lt;/iframe&gt;'
        )
        self.assertTrue(pattern.match(rendered))

    @unittest.skipUnless(
        markdown and markdown_version >= (2, 1),
        'markdown >= 2.1 not installed'
    )
    def test_markdown_attribute_disable(self):
        t = Template("{% load markup %}{{ markdown_content|markdown:'safe' }}")
        markdown_content = "{@onclick=alert('hi')}some paragraph"
        rendered = t.render(Context(
            {'markdown_content': markdown_content})).strip()
        self.assertTrue('@' in rendered)

    @unittest.skipUnless(
        markdown and markdown_version >= (2, 1),
        'markdown >= 2.1 not installed'
    )
    def test_markdown_attribute_enable(self):
        t = Template("{% load markup %}{{ markdown_content|markdown }}")
        markdown_content = "{@onclick=alert('hi')}some paragraph"
        rendered = t.render(Context(
            {'markdown_content': markdown_content})).strip()
        self.assertFalse('@' in rendered)

    @unittest.skipIf(markdown, 'markdown is installed')
    def test_no_markdown(self):
        t = Template("{% load markup %}{{ markdown_content|markdown }}")
        rendered = t.render(Context(
            {'markdown_content': self.markdown_content})).strip()
        self.assertEqual(rendered, self.markdown_content)

    @unittest.skipUnless(commonmark, 'commonmark not installed')
    def test_commonmark(self):
        t = Template("{% load markup %}{{ markdown_content|commonmark }}")
        rendered = t.render(
            Context({'markdown_content': self.markdown_content})).strip()
        pattern = re.compile("""<p>Paragraph 1\s*</p>\s*<h2>\s*An h2</h2>""")
        self.assertTrue(pattern.match(rendered))

    @unittest.skipUnless(commonmark, 'commonmark not installed')
    def test_commonmark_html_code(self):
        t = Template("{% load markup %}{{ markdown_content|commonmark }}")
        rendered = t.render(Context({
            'markdown_content': self.markdown_content_with_html_code
        })).strip()
        pattern = re.compile(
            '<p>Paragraph 1\s*</p>\s*<h2>\s*An h2</h2>' +
            '\s*<pre><code>\s*&lt;video width=&quot;320&quot'
        )
        self.assertTrue(pattern.match(rendered))

    @unittest.skipUnless(commonmark, 'commonmark not installed')
    def test_commonmark_html_iframe_code(self):
        t = Template("{% load markup %}{{ markdown_content|commonmark }}")
        rendered = t.render(Context({
            'markdown_content': self.markdown_content_with_iframe_code
        })).strip()
        pattern = re.compile(
            '<p>Paragraph 1\s*</p>\s*<h2>\s*An h2</h2>' +
            '\s*<pre><code>\s*&lt;iframe ' +
            'src=&quot;http://example.com&quot;&gt;' +
            '&lt;/iframe&gt;'
        )
        self.assertTrue(pattern.match(rendered))

    @unittest.skipUnless(commonmark, 'commonmark not installed')
    def test_commonmark_empty_str(self):
        t = Template("{% load markup %}{{ markdown_content|commonmark }}")
        rendered = t.render(Context({'markdown_content': ''})).strip()
        self.assertEqual(rendered, '')

    @unittest.skipUnless(commonmark, 'commonmark not installed')
    def test_commonmark_none(self):
        t = Template("{% load markup %}{{ markdown_content|commonmark }}")
        rendered = t.render(Context({'markdown_content': None})).strip()
        self.assertEqual(rendered, '<p>None</p>')

    @unittest.skipUnless(docutils, 'docutils not installed')
    def test_docutils(self):
        t = Template("{% load markup %}{{ rest_content|restructuredtext }}")
        rendered = t.render(Context({
            'rest_content': self.rest_content
        })).strip()
        # Different versions of docutils return slightly different HTML
        try:
            # Docutils v0.4 and earlier
            self.assertEqual(
                rendered,
                '<p>Paragraph 1</p>\n' +
                '<p>Paragraph 2 with a <a class="reference" ' +
                'href="http://www.example.com/">link</a></p>')
        except AssertionError:
            # Docutils from SVN (which will become 0.5)
            self.assertEqual(
                rendered,
                '<p>Paragraph 1</p>\n' +
                '<p>Paragraph 2 with a ' +
                '<a class="reference external" ' +
                'href="http://www.example.com/">link</a></p>')

    @unittest.skipIf(docutils, 'docutils is installed')
    def test_no_docutils(self):
        t = Template("{% load markup %}{{ rest_content|restructuredtext }}")
        rendered = t.render(
            Context({'rest_content': self.rest_content})).strip()
        self.assertEqual(rendered, self.rest_content)



"""
Set of "markup" template filters for Django.
These filters transform plain text
markup syntaxes to HTML; currently there is support for:

    * Textile, which requires the PyTextile library available at
      http://loopcore.com/python-textile/

    * Markdown, which requires the Python-markdown library from
      http://www.freewisdom.org/projects/python-markdown

    * reStructuredText, which requires docutils from http://docutils.sf.net/

    * CommonMark, which requires commonmark http://commonmark.org/
"""

from django import template
from django.conf import settings
from django.utils.encoding import smart_str, force_text
from django.utils.safestring import mark_safe

register = template.Library()


@register.filter(is_safe=True)
def textile(value):
    """
    :type value: str

    :rtype: str
    """
    import textile

    return mark_safe(force_text(
        textile.textile(smart_str(value)))
    )


@register.filter(is_safe=True)
def markdown(value, args=''):
    """
    Runs Markdown over a given value, optionally using various
    extensions python-markdown supports.

    Syntax::

        {{ value|markdown:"extension1_name,extension2_name..." }}

    To enable safe mode, which strips raw HTML and only returns HTML
    generated by actual Markdown syntax, pass "safe" as the first
    extension in the list.

    If the version of Markdown in use does not support extensions,
    they will be silently ignored.

    :type value: str
    :type args: str

    :rtype: str
    """
    import markdown

    extensions = [e for e in args.split(',') if e]
    if len(extensions) > 0 and extensions[0] == "safe":
        extensions = extensions[1:]
        safe_mode = True
    else:
        safe_mode = False

    return mark_safe(markdown.markdown(
        force_text(value),
        extensions,
        safe_mode=safe_mode,
        enable_attributes=(not safe_mode)
    ))


@register.filter(is_safe=True)
def commonmark(value):
    """
    Runs commonmark over a given value.

    Syntax::

        {{ value|commonmark }}

    :type value: str

    :rtype: str
    """
    import commonmark

    parser = commonmark.Parser()
    renderer = commonmark.HtmlRenderer()
    ast = parser.parse(force_text(value))
    return mark_safe(
        force_text(renderer.render(ast))
    )


@register.filter(is_safe=True)
def restructuredtext(value):
    """
    :type value: str
    :rtype: str
    """
    from docutils.core import publish_parts

    docutils_settings = getattr(
        settings,
        "RESTRUCTUREDTEXT_FILTER_SETTINGS",
        {}
    )
    parts = publish_parts(
        source=smart_str(value),
        writer_name="html4css1",
        settings_overrides=docutils_settings
    )
    return mark_safe(force_text(parts["fragment"]))



import io
from setuptools import setup
from os import path

this_directory = path.abspath(path.dirname(__file__))
with io.open(path.join(this_directory, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-markwhat',
    version=".".join(map(str, __import__('django_markwhat').__version__)),
    packages=['django_markwhat', 'django_markwhat.templatetags'],
    url='http://pypi.python.org/pypi/django-markwhat',
    license="BSD-3-Clause",
    author='Alireza Savand',
    author_email='alireza.savand@gmail.com',
    install_requires=['Django', ],
    description="A collection of template filters that implement " +
            "common markup languages.",
    long_description=long_description,
    long_description_content_type='text/x-rst',
    keywords=[
        'django',
        'markdown',
        'markup',
        'textile',
        'rst',
        'reStructuredText',
        'docutils',
        'commonmark',
        'web'
    ],
    platforms='OS Independent',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Development Status :: 6 - Mature",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
    ],
)
